@@ -27,10 +27,10 @@ from bzt import ToolError
 
 from bzt.utils import numeric_types, Environment, RequiredTool, PIPE, SoapUIScriptConverter
 from bzt.utils import to_json, BetterDict, ensure_is_dict, dehumanize_time
-
-from .templates import FileLister
+from bzt.environment_helpers import expand_envs_with_os
 from .dicts import Scenario
 from .names import EXEC, SCENARIO
+from .templates import HavingInstallableTools
 
 
 class EngineModule(object):
@@ -46,6 +46,7 @@ class EngineModule(object):
         self.engine = None
         self.settings = BetterDict()
         self.parameters = BetterDict()
+        self.errors = None
 
     def prepare(self):
         """
@@ -163,14 +164,18 @@ class Aggregator(EngineModule):
         super(Aggregator, self).__init__()
         self.is_functional = is_functional
 
+    @staticmethod
+    def converter(data):
+        return data
 
-class ScenarioExecutor(EngineModule):
+
+class ScenarioExecutor(EngineModule, HavingInstallableTools):
     """
     :type provisioning: engine.Provisioning
     :type execution: BetterDict
     """
 
-    EXEC = EXEC     # backward compatibility
+    EXEC = EXEC  # backward compatibility
     RAMP_UP = "ramp-up"
     HOLD_FOR = "hold-for"
     CONCURR = "concurrency"
@@ -182,7 +187,7 @@ class ScenarioExecutor(EngineModule):
         super(ScenarioExecutor, self).__init__()
         self.env = Environment(log=self.log)
         self.provisioning = None
-        self.execution = BetterDict()  # FIXME: why have this field if we have `parameters` from base class?
+        self.execution = BetterDict()
         self._cached_scenario = None
         self.label = None
         self.widget = None
@@ -405,10 +410,17 @@ class ScenarioExecutor(EngineModule):
         return self.LOAD_FMT(concurrency=concurrency, ramp_up=ramp_up, throughput=throughput, hold=hold,
                              iterations=iterations, duration=duration, steps=steps)
 
+    def resource_files(self):
+        """
+        Get list of resource files
+
+        :rtype: list
+        """
+        return []
+
     def get_resource_files(self):
         files_list = []
-        if isinstance(self, FileLister):
-            files_list.extend(self.resource_files())
+        files_list.extend(self.resource_files())
         files_list.extend(self.execution.get("files", []))
         return files_list
 
@@ -417,7 +429,15 @@ class ScenarioExecutor(EngineModule):
 
     def prepare(self):
         super(ScenarioExecutor, self).prepare()
-        self.env.set(self.execution.get("env"))
+        self.env.set(expand_envs_with_os(self.execution.get("env")))
+
+    def get_widget(self):
+        """
+        Returns widget instance to be added to sidebar
+
+        :rtype: urwid.Widget
+        """
+        pass
 
     def _execute(self, args, **kwargs):
         self.preprocess_args(args)
@@ -437,10 +457,15 @@ class ScenarioExecutor(EngineModule):
             raise ToolError("Failed to start %s: %s (%s)" % (self.__class__.__name__, exc, args))
         return process
 
+    def get_error_diagnostics(self):
+        """
+        :rtype: list[str]
+        """
+        return None
+
     def post_process(self):
         if self.stdout:
             self.stdout.close()
         if self.stderr:
             self.stderr.close()
         super(ScenarioExecutor, self).post_process()
-
