@@ -372,7 +372,7 @@ class JMX(object):
         proxy.append(JMX._bool_prop("HTTPSampler.follow_redirects", follow_redirects))
         proxy.append(JMX._bool_prop("HTTPSampler.auto_redirects", False))
 
-        if timeout is not None:
+        if timeout:
             proxy.append(JMX._string_prop("HTTPSampler.connect_timeout", timeout))
             proxy.append(JMX._string_prop("HTTPSampler.response_timeout", timeout))
 
@@ -905,18 +905,6 @@ class JMX(object):
 
         return cfg
 
-    @staticmethod
-    def _get_dur_assertion(timeout):
-        """
-
-        :type timeout: int
-        :return:
-        """
-        element = etree.Element("DurationAssertion", guiclass="DurationAssertionGui",
-                                testclass="DurationAssertion", testname="Timeout Check")
-        element.append(JMX._string_prop("DurationAssertion.duration", timeout))
-        return element
-
     @staticmethod
     def get_constant_timer(delay):
         timer_type = "ConstantTimer"
@@ -1012,7 +1000,7 @@ class JMX(object):
                                 testclass="BoundaryExtractor", testname="Get %s" % varname, enabled="true")
         element.append(JMX._string_prop("BoundaryExtractor.useHeaders", subject))
         element.append(JMX._string_prop("BoundaryExtractor.refname", varname))
-        element.append(JMX._string_prop("BoundaryExtractor.lboundary", left))  # TODO: html-escape boundaries?
+        element.append(JMX._string_prop("BoundaryExtractor.lboundary", left))
         element.append(JMX._string_prop("BoundaryExtractor.rboundary", right))
         element.append(JMX._string_prop("RegexExtractor.default", defvalue))
         element.append(JMX._string_prop("RegexExtractor.match_number", match_no))
@@ -1373,7 +1361,6 @@ class JMX(object):
 
     @staticmethod
     def _get_foreach_controller(input_var, loop_var):
-        # TODO: useSeparator option
         controller = etree.Element("ForeachController", guiclass="ForeachControlPanel", testclass="ForeachController",
                                    testname="ForEach Controller")
         controller.append(JMX._string_prop("ForeachController.inputVal", input_var))
