@@ -19,7 +19,6 @@ from os import listdir
 from os.path import join
 
 from bzt import ToolError
-from bzt.engine import HavingInstallableTools
 from bzt.modules import SubprocessedExecutor
 from bzt.modules.functional import FuncSamplesReader
 from bzt.modules.jmeter import JTLReader
@@ -29,7 +28,7 @@ from .tools import JUnitPlatformCommons, JUnitPlatformLauncher, JUnitPlatformEng
 from .tools import JUnitPlatformSuiteApi, JUnitVintageEngine, ApiGuardian, JUnit, OpenTest4j, TestNG
 
 
-class JavaTestRunner(SubprocessedExecutor, HavingInstallableTools):
+class JavaTestRunner(SubprocessedExecutor):
     """
     Allows to test java and jar files
     :type script: str
@@ -57,18 +56,12 @@ class JavaTestRunner(SubprocessedExecutor, HavingInstallableTools):
 
         self._tools.append(self._get_tool(TclLibrary))
 
-        self.tool = self._get_tool(JavaVM, mandatory=True)
+        self.tool = self._get_tool(JavaVM)
         self._tools.append(self.tool)
 
         self._check_tools(self._tools)
 
     def _add_jar_tool(self, req_tool_class, **kwargs):
-        # todo: it's for backward compatibility only, remove it later
-        if "local_path" in kwargs:
-            local_path = kwargs.pop("local_path")
-            if local_path:
-                kwargs["config"] = BetterDict.from_dict({"config": local_path})
-
         req_tool = self._get_tool(req_tool_class, **kwargs)
         self._tools.append(req_tool)
         self.class_path.append(req_tool.tool_path)
@@ -306,26 +299,25 @@ class TestNGTester(JavaTestRunner):
     __test__ = False  # Hello, nosetests discovery mechanism
 
     def install_required_tools(self):
-        self._add_jar_tool(TestNG, local_path=self.settings.get("path"))
+        local_path = self.settings.get("path", None)
+        config = {}
+        if local_path:
+            config = BetterDict.from_dict({"config": local_path})
+        self._add_jar_tool(TestNG, config=config)
         super(TestNGTester, self).install_required_tools()
 
-    def detected_testng_xml(self):
-        script_path = self.get_script_path()
-        if script_path and self.settings.get("autodetect-xml", True):
-            script_dir = get_full_path(script_path, step_up=1)
-            testng_xml = join(script_dir, 'testng.xml')
+    def _store_testng_xml(self):
+        if not self.execution.get('testng-xml') and self.settings.get("autodetect-xml", True):
+            script_dir = get_full_path(self.get_script_path(), step_up=1)
+            testng_xml = os.path.join(script_dir, 'testng.xml')
             if os.path.exists(testng_xml):
-                return testng_xml
-        return None
+                self.log.info("Detected testng.xml file at %s", testng_xml)
+                self.execution['testng-xml'] = testng_xml
 
     def resource_files(self):
         resources = super(TestNGTester, self).resource_files()
+        self._store_testng_xml()
         testng_xml = self.execution.get('testng-xml')
-        if not testng_xml:
-            testng_xml = self.detected_testng_xml()
-            if testng_xml:
-                self.log.info("Detected testng.xml file at %s", testng_xml)
-                self.execution['testng-xml'] = testng_xml
         if testng_xml:
             resources.append(testng_xml)
 
@@ -339,6 +331,7 @@ class TestNGTester(JavaTestRunner):
         jar_list = [join(self.working_dir, jar) for jar in listdir(self.working_dir) if jar.endswith(".jar")]
         jar_list.extend(self._collect_script_files({".jar"}))
         self.class_path.extend(jar_list)
+        self._store_testng_xml()
 
         with open(self.props_file, 'wt') as props:
             props.write("report_file=%s\n" % self.report_file)
@@ -353,7 +346,7 @@ class TestNGTester(JavaTestRunner):
             for index, item in enumerate(jar_list):
                 props.write("target_%s=%s\n" % (index, item.replace(os.path.sep, '/')))
 
-            testng_xml = self.execution.get('testng-xml') or self.detected_testng_xml()
+            testng_xml = self.execution.get('testng-xml')
             if testng_xml:
                 props.write('testng_config=%s\n' % testng_xml.replace(os.path.sep, '/'))
 
