@@ -25,6 +25,7 @@ import itertools
 import json
 import locale
 import logging
+import math
 import mimetypes
 import operator
 import os
@@ -46,25 +47,27 @@ from abc import abstractmethod
 from collections import defaultdict, Counter
 from contextlib import contextmanager
 from distutils.version import LooseVersion
+from io import IOBase
+from ssl import SSLError
 from subprocess import CalledProcessError, PIPE, check_output, STDOUT
+from urllib import parse
+from urllib.error import URLError
+from urllib.request import url2pathname
 from webbrowser import GenericBrowser
 
-import math
 import psutil
 import requests
 import requests.adapters
-from io import IOBase
 from lxml import etree
 from progressbar import ProgressBar, Percentage, Bar, ETA
-from urllib import parse
-from urllib.request import url2pathname
+from requests.exceptions import ReadTimeout
 from urwid import BaseScreen
 
 from bzt import TaurusInternalException, TaurusNetworkError, ToolError, TaurusConfigError
 
-
 LOG = logging.getLogger("")
 CALL_PROBLEMS = (CalledProcessError, OSError)
+NETWORK_PROBLEMS = (IOError, URLError, SSLError, ReadTimeout, TaurusNetworkError)
 numeric_types = (int, float, complex)
 viewvalues = operator.methodcaller("values")
 
@@ -76,7 +79,7 @@ def unicode_decode(string, errors="strict"):
         return string
 
 
-def communicate(proc):  # todo: replace usage of it with sync_run()
+def communicate(proc):
     out, err = proc.communicate()
     out = unicode_decode(out, errors="ignore")
     err = unicode_decode(err, errors="ignore")
@@ -109,6 +112,7 @@ def stream_decode(string):
     else:
         return string
 
+
 def sync_run(args, env=None):
     output = check_output(args, env=env, stderr=STDOUT)
     return stream_decode(output).rstrip()
@@ -338,7 +342,6 @@ class BetterDict(defaultdict):
                 key = key[1:]
 
             if prefix == "^":  # eliminate flag
-                # TODO: improve logic - use val contents to see what to eliminate
                 if key in self:
                     self.pop(key)
                 continue
@@ -383,7 +386,6 @@ class BetterDict(defaultdict):
                     if isinstance(lefty, BetterDict) and isinstance(righty, BetterDict):
                         lefty.merge(righty)
                     else:
-                        # todo: should we log all overwriting cases?
                         LOG.warning("Overwriting the value of %r when merging configs", key)
                         left[index] = righty
                 else:
@@ -424,23 +426,23 @@ class BetterDict(defaultdict):
         keys = set(self.keys())
         for key in keys:
             ikey = "!" + key
-            if (key in rules) or (ikey in rules):   # we have rule for this key
+            if (key in rules) or (ikey in rules):  # we have rule for this key
                 current_black_list = black_list if key in rules else not black_list
                 rkey = key if key in rules else ikey
 
                 if isinstance(rules.get(rkey), dict):
-                    if isinstance(self.get(key), BetterDict):       # need to go deeper
+                    if isinstance(self.get(key), BetterDict):  # need to go deeper
                         self.get(key).filter(rules[rkey], black_list=current_black_list)
                     elif not current_black_list:
                         del self[key]
                 elif current_black_list:
-                    del self[key]   # must be blacklisted
+                    del self[key]  # must be blacklisted
             elif not black_list:
-                del self[key]       # remove unknown key
+                del self[key]  # remove unknown key
 
             current = self.get(key, None)
             if isinstance(current, (dict, list)) and not current:
-                del self[key]       # clean empty
+                del self[key]  # clean empty
 
     def __repr__(self):
         return dict(self).__repr__()
@@ -529,7 +531,6 @@ def shell_exec(args, cwd=None, stdout=PIPE, stderr=PIPE, stdin=PIPE, shell=False
         if pgrp:
             kwargs["preexec_fn"] = os.setpgrp
         return psutil.Popen(args, **kwargs)
-        # FIXME: shouldn't we bother closing opened descriptors?
 
 
 class Environment(object):
@@ -745,10 +746,10 @@ def ensure_is_dict(container, key, sub_key):
     """
     if isinstance(container, BetterDict):
         container.get(key, force_set=True)
-    elif isinstance(container, dict):   # todo: remove after fixing merge
+    elif isinstance(container, dict):  # todo: remove after fixing merge
         container[key] = BetterDict()
 
-    if not isinstance(container[key], dict):    # todo: replace dict with BetterDict after fixing merge
+    if not isinstance(container[key], dict):  # todo: replace dict with BetterDict after fixing merge
         container[key] = BetterDict.from_dict({sub_key: container[key]})
 
     return container[key]
@@ -865,7 +866,6 @@ def to_json(obj, indent=True):
     :param obj: object to convert
     :return:
     """
-    # NOTE: you can set allow_nan=False to fail when serializing NaN/Infinity
     return json.dumps(obj, indent=indent, cls=ComplexEncoder)
 
 
@@ -887,7 +887,6 @@ class ComplexEncoder(json.JSONEncoder):
     """
     Magic class to help serialize in JSON any object.
     """
-    # todo: should we add complex type?
     TYPES = (dict, list, tuple, str, int, float, bool, type(None))
 
     def default(self, obj):  # pylint: disable=method-hidden
@@ -1066,13 +1065,26 @@ def is_int(str_val):
         return False
 
 
-def shutdown_process(process_obj, log_obj):
-    count = 60
-    while process_obj and process_obj.poll() is None:
+def shutdown_process(process_obj, log_obj, send_sigterm=True):
+    # unhandled sigterm causes immediate break of the process
+    # so it makes sense to turn send_sigterm off for graceful (or delayed) shutdown
+    time_limit = 60
+    for count in range(time_limit + 1):
         time.sleep(1)
-        count -= 1
-        kill_signal = signal.SIGTERM if count > 0 else signal.SIGKILL
-        log_obj.info("Terminating process PID %s with signal %s (%s tries left)", process_obj.pid, kill_signal, count)
+
+        process_still_works = process_obj and process_obj.poll() is None
+        if not process_still_works:
+            break
+
+        if count < time_limit and not send_sigterm:
+            continue  # time for sigterm, but it isn't supported (unhandled)
+        elif count == time_limit and not is_windows():
+            kill_signal = signal.SIGKILL  # send KILL to program on linux/mac
+        else:
+            kill_signal = signal.SIGTERM  # KILL doesn't supported on win, send TERM instead
+
+        msg = "Terminating process PID %s with signal %s (%s tries left)"
+        log_obj.info(msg, process_obj.pid, kill_signal, time_limit - count)
         try:
             if is_windows():
                 cur_pids = psutil.pids()
@@ -1097,7 +1109,7 @@ class LocalFileAdapter(requests.adapters.BaseAdapter):
     def _chkpath(method, path):
         """Return an HTTP status for the given filesystem path."""
         if method.lower() in ('put', 'delete'):
-            return 501, "Not Implemented"  # TODO
+            return 501, "Not Implemented"
         elif method.lower() not in ('get', 'head'):
             return 405, "Method Not Allowed"
         elif os.path.isdir(path):
@@ -1237,7 +1249,7 @@ class ExceptionalDownloader(object):
         super(ExceptionalDownloader, self).__init__()
         self.http_client = http_client
 
-    def get(self, url, filename=None, reporthook=None, data=None, suffix="", timeout=5.0):
+    def get(self, url, filename=None, reporthook=None, data=None, suffix="", timeout=30.0):
         if os.getenv("TAURUS_DISABLE_DOWNLOADS", ""):
             raise TaurusInternalException("Downloads are disabled by TAURUS_DISABLE_DOWNLOADS env var")
 
@@ -1256,26 +1268,20 @@ class RequiredTool(object):
     """
     Abstract required tool
     """
+    VERSION = None
+    DOWNLOAD_LINK = None
 
     def __init__(self, log=None, tool_path="", download_link="", http_client=None,
-                 env=None, version=None, installable=True, mandatory=True):
+                 env=None, version=None, installable=True, mandatory=True, dry_install=False):
         self.http_client = http_client
         self.tool_path = os.path.expanduser(tool_path)
-        self.download_link = download_link
+        self.download_link = download_link or self.DOWNLOAD_LINK
         self.mirror_manager = None
         self.mandatory = mandatory
-
-        self.version = None
-        if version is not None:
-            self.version = str(version)
-
         self.installable = installable
-
+        self.version = str(version) if version else self.VERSION
         self.tool_name = self.__class__.__name__
-
-        # for browsermobproxy compatability, remove it later
-        if not isinstance(log, logging.Logger):
-            log = None
+        self.dry_install = dry_install
 
         log = log or LOG
         self.log = log.getChild(self.tool_name)
@@ -1298,6 +1304,10 @@ class RequiredTool(object):
         return False
 
     def install(self):
+        if self.dry_install:
+            self.log.info(f"Dry installation for {self.tool_name}")
+            return
+
         if not self.installable:
             msg = "%s isn't found, automatic installation isn't implemented" % self.tool_name
             if self.mandatory:
@@ -1334,13 +1344,15 @@ class RequiredTool(object):
                     raise
                 except BaseException as exc:
                     self.log.error("Error while downloading %s: %s" % (link, exc))
-        raise TaurusInternalException("%s download failed: No more links to try" % self.tool_name)
+        error_message = "%s download failed: No more links to try" % self.tool_name
+        if self.mandatory:
+            raise TaurusInternalException(error_message)
+        else:
+            self.log.warning(error_message)
 
 
 class JavaVM(RequiredTool):
     def __init__(self, **kwargs):
-        if "mandatory" not in kwargs:
-            kwargs["mandatory"] = False
         super(JavaVM, self).__init__(installable=False, tool_path="java", **kwargs)
 
     def _get_version(self, output):
@@ -1813,6 +1825,15 @@ def parse_think_time(think_time, full=False):
         return [res.group(i + 1).lower() for i in range(3)]
 
 
+def convert_body_to_string(request):
+    if isinstance(request.body, (dict, list, numeric_types)):
+        if request.get_header('content-type') == 'application/json' or isinstance(request.body, numeric_types):
+            request.body = json.dumps(request.body)
+        elif not simple_body_dict(request.body):
+            LOG.debug('Header "Content-Type: application/json" is required for body: "%s"', request.body)
+            request.body = json.dumps(request.body)
+
+
 class SoapUIScriptConverter(object):
     NAMESPACES = dict(con="http://eviware.com/soapui/config")
 
@@ -2193,7 +2214,7 @@ class SoapUIScriptConverter(object):
         case_properties = {
             "#TestCase#" + key: value
             for key, value in iteritems(case_properties)
-            }
+        }
         case_level_props = BetterDict.from_dict(suite_level_props)
         case_level_props.merge(case_properties)
 
